local file = {priority = 1}

Enemy = {list = {}}

function Enemy:new(pos, radius, health)
	local self = Class:init(Enemy,Entity,{pos})

	self.radius = radius or 4
	self.health = health or 3
	self.maxHealth = self.health
	self.sprite = Sprite:new()
	self.sprite:setParent(self)
	self.hitTimer = 0
	self.hitTimerSpeed = 4
	self.flip = 1
	self.size = 1
	self.bloodAmount = 40

	self.hitSound = la.newSource("assets/audio/enemyhit.wav","static")
	self.deathSound = la.newSource("assets/audio/enemydeath.wav","static")
	self.impactSound = la.newSource("assets/audio/impact.wav","static")
	self.watchoutSound = la.newSource("assets/audio/watchout.wav","static")
	self.projectileSound = la.newSource("assets/audio/projectile.wav","static")

	self.sprite.order = 0.9

	table.insert(Enemy.list,self)
	return self
end


function Enemy:collides(offset)
	offset = offset or {0,0}
	local pos = self:getCenter()
	for a = 0,rad(360),rad(20) do
		if World:getCell(pos[1]+cos(a)*self.radius + offset[1], pos[2]+sin(a)*self.radius + offset[2]).solid then
			return true
		end
	end
	return false
end

function Enemy:damage(damage)
	if not self.justDamaged then
		self.health = self.health - (damage or 1)
		self.hitTimer = 1
		local playerXDir = sign(self.pos[1]-Player.pos[1])
		if self.health <= 0 then
			Corpse:new(self.anim:getFrame(), self.pos, {(30+random()*10)*playerXDir,-random()*10-120}, playerXDir)
			Juice:freeze(1/30)
			Juice:zoom(0.9)
			Juice:shake(2)
			Utils.playSound(self.deathSound,1.3,1+random()*rsign()*0.1)

			for i=1,self.bloodAmount do
				local a = rad(270) + rad(90)*random()*rsign()
				local speed = random()*80
				local particle = Particle:new(self:getCenter(),1.5,{5/8,5/8,5/8},1,150,{cos(a)*speed,sin(a)*speed})
				particle.order = 0.8
			end

			if self.isBoss then
				Key:new(self:getCenter())
				AudioHandler.boss:stop()
				for i, p in pairs(Projectile.list) do
					p:destroy()
				end
			end

			self:destroy()
		end
		self.justDamaged = true

		for i=1,15 do
			local a = rad(90)*playerXDir*-1+rad(90) + rad(20)*random()*rsign()
			local speed = random()*170
			local particle = Particle:new(self.pos,1.25,{5/8,5/8,5/8},1,150,{cos(a)*speed,sin(a)*speed-speed*0.2})
			particle.order = 0.8
		end

		if self.health > 0 then
			Utils.playSound(self.hitSound,1,1+random()*rsign()*0.1)

			Juice:freeze(1/60)
			Juice:shake(1)
			Juice:zoom(0.98)
		end
	end
end

function Enemy:getCenter()
	return {self.pos[1],self.pos[2]-self.radius}
end


function Enemy:updateHit()
	self.hitTimer = max(0, self.hitTimer - self.hitTimerSpeed*gdt)
	self.sprite.fullBright = self.hitTimer > 0

	local hit = (self.hitTimer^0.75) * 0.4
	self.sprite.sx = self.flip * self.size + hit*self.flip
	self.sprite.sy = self.size - hit
	self.justDamaged = false
end

function Enemy:damagePlayerOnContact()
	if math.dist(self.pos[1],self.pos[2], Player.pos[1],Player.pos[2]) < self.radius + Player.radius -2 then
		Player:damage(1)
	end
end

function Enemy:setSprite()
	if self.anim then
		self.sprite.img = self.anim:getFrame()
	end
end

function Enemy:setDefaults()
	self.maxHealth = self.maxHealth or self.health or 1
end

function Enemy:killOffScreen()
	if not self.isBoss and not self:onScreen() then
		self:destroy()
	end
end

function Enemy:update()
	self:setDefaults()
	self:updateHit()
	self:damagePlayerOnContact()
	self:setSprite()
	self:killOffScreen()
end


function Enemy:destroy()
	Entity.destroy(self)
	self.sprite:destroy()
	removeFromTable(Enemy.list,self)
end

function file:init()
end

function file:update()
	for i,e in pairs(Enemy.list) do
		if e.parent then
			local parentPos = e.parent.pos

			if self.parentFloored then
				parentPos = {floor(parentPos[1]),floor(parentPos[2])}
			end

			e:setPos(parentPos)
			e:move(e.parentOffset)
		end
	end
end

function file:draw()
end

-- [[ RETURN ]] --
return file