local file = {priority = 11}

Hud = {}

function Hud:new()
	local self = Class:init(Hud,Dynamic)
	
	self.healthIcon = lg.newImage("assets/hud/health.png")
	self.sacrificeIcon = lg.newImage("assets/hud/sacrifice.png")
	self.selectIcon = lg.newImage("assets/hud/select.png")
	self.keyIcon = lg.newImage("assets/hud/key.png")
	self.sacIcon = lg.newImage("assets/hud/sacrifices.png")

	self.w,self.h = Screen.resolution[1],Screen.resolution[2]
	self.order = 1000
	self.endFade = 0
	self.gotoDoorTimer = 0

	return self
end


function Hud:apply()
	lg.origin()
end

function Hud:drawHealth()
	local w,h = 30,6
	local x,y = 5,self.h-5-h
	local space = 10

	lg.setColor(1,1,1)
	for i=0,Player.health-1 do
		lg.draw(self.healthIcon,x+i*space,y)
	end
end

function Hud:drawKeys()
	local x,y = 24,self.h-5-18
	lg.draw(self.keyIcon,x-20,y+2)
	lg.print(Player.keys.."/3",x-10,y)
end

function Hud:drawSacrifices()
	local x,y = 50,self.h-5-18
	lg.draw(self.sacIcon,x-20,y+2)
	lg.print(":"..Player.sacrifices,x-10,y)
end

function Hud:drawGotoDoorText()
	if Player.keys == 3 and Game.state == "playing" then
		self.gotoDoorTimer = self.gotoDoorTimer - gdt * 0.2
		if self.gotoDoorTimer > 0 then
			lg.setColor(1,1,1)
			lg.printf("YOU HAVE ALL 3 KEYS. RETURN TO THE GREAT DOOR.", self.w*0.5-100,50,200,"center")
		end
	else
		self.gotoDoorTimer = 1
	end
end

function Hud:drawSacrificeSelection()
	local y = 30
	local dy = 110

	if Game.state == "sacrifice" then
		lg.setColor(0,0,0,0.2)
		lg.rectangle("fill",0,y,self.w,120)

		lg.setColor(1,1,1)
		lg.printf("OBTAINED KEY! ITS EVIL PRESENCE FORCES YOU TO MAKE A SACRIFICE.", self.w*0.5-100,y,200,"center")
		lg.setColor(5.5/8,5.5/8,5.5/8)
		lg.printf("SACRIFICE: "..SacrificeHandler.chooseList[SacrificeHandler.chooseIndex].name, self.w*0.5-500,y+25,1000,"center")

		local space = 50
		
		lg.setColor(1,1,1)
		lg.draw(self.selectIcon,self.w*0.5 - space*1.5,y+65, -rad(90), 1,1, floor(self.selectIcon:getWidth()*0.5),self.selectIcon:getHeight())
		lg.draw(self.selectIcon,self.w*0.5 + space*1.5,y+65, rad(90), 1,1, floor(self.selectIcon:getWidth()*0.5),self.selectIcon:getHeight())
		for i=1,3 do
			local selectOffset = 0
			if i == SacrificeHandler.chooseIndex then
				selectOffset = 10
				lg.draw(self.selectIcon,self.w*0.5 + (i-2) * space,y+65, 0, 1,1, floor(self.selectIcon:getWidth()*0.5),self.selectIcon:getHeight())
			end
			lg.draw(self.sacrificeIcon,self.w*0.5 + (i-2) * space,y+65 - selectOffset, 0, 1,1, self.sacrificeIcon:getWidth()*0.5,self.sacrificeIcon:getHeight())
		end

		lg.setColor(1,1,1)
		lg.printf(SacrificeHandler.chooseList[SacrificeHandler.chooseIndex].description, self.w*0.5-500,dy,1000,"center")
		lg.printf("CONFIRM: 'SPACE'", self.w*0.5-500,dy+25,1000,"center")
	end
end

function Hud:drawGameOverScreen()
	if Game.state == "gameover" then
		lg.setColor(0,0,0,0.2)
		lg.rectangle("fill",0,0,10000,1000)

		lg.setColor(1,1,1)
		lg.printf("YOU DIED!", self.w*0.5-500,60,1000,"center")
		lg.printf("PRESS 'R' TO RESPAWN", self.w*0.5-500,120,1000,"center")

		if lk.isDown("r") then
			Player:respawn()
		end
	end
end

function Hud:drawEndScreen()
	if Game.state == "end" then
		self.endFade = self.endFade + gdt

		lg.setColor(2/8,2/8,2/8,self.endFade)
		lg.rectangle("fill",0,0,10000,1000)

		if self.endFade > 3 then
			lg.setColor(5/8,5/8,5/8)
			lg.printf("YOU DID IT!", self.w*0.5-150,30,300,"center")
		end

		if self.endFade > 7 then
			lg.setColor(5/8,5/8,5/8)
			lg.printf("WITH THE 3 KEYS IN HAND YOU OPEN THE GREAT DOOR WITH EASE.", self.w*0.5-150,50,300,"center")
		end

		if self.endFade > 12 then
			lg.setColor(5/8,5/8,5/8)
			lg.printf("BUT THE GREAT DOOR ONLY LEADS TO GREATER HARDSHIPS.", self.w*0.5-150,70,300,"center")
		end

		if self.endFade > 17 then
			lg.setColor(7/8,7/8,7/8)
			lg.printf("KEEP YOUR SACRIFICES. FACE EVIL ONCE MORE.", self.w*0.5-150,90,300,"center")
		end

		if self.endFade > 22 then
			lg.setColor(5/8,5/8,5/8)
			lg.printf("PRESS 'R' TO KEEP GOING", self.w*0.5-150,130,300,"center")

			if lk.isDown("r") then
				Game:spawnBosses()
				Game.state = "playing"
				Player:setPos(Player.spawnPoint)
				Player:setCamera()
				Player.keys = 0
				Camera:setPos(Camera.chasePos)
				SacrificeHandler.bossTimerSpeed = SacrificeHandler.bossTimerSpeed + 0.75
				Player.currentRun = Player.currentRun + 1

				if Player.sacrifices/3 == 1 then
					shader_screen:send("palette", lg.newImage("assets/palettes/2.png"))
				end
				if Player.sacrifices/3 == 2 then
					shader_screen:send("palette", lg.newImage("assets/palettes/3.png"))
				end
				if Player.sacrifices/3 == 3 then
					shader_screen:send("palette", lg.newImage("assets/palettes/4.png"))
				end

			end
		end
	else
		self.endFade = 0
	end
end

function Hud:drawBossHealth()
	for i, e in pairs(Enemy.list) do
		if e:onScreen() and e.isBoss then
			local w,h = self.w * 0.9,8
			local x,y = self.w*0.5-w*0.5,5

			lg.setColor(2/8,2/8,2/8)
			lg.rectangle("fill",x,y,w,h)

			lg.setColor(3/8,3/8,3/8)
			lg.rectangle("fill",x+1,y+1,w-2,h-2)

			lg.setColor(5/8,5/8,5/8)
			lg.rectangle("fill",x+1,y+1,(w-2)*e.health/e.maxHealth,h-2)
		end
	end
end

function Hud:draw(dontApply)
	if not dontApply then
		self:apply()
	end
	self:drawHealth()
	self:drawKeys()
	self:drawSacrifices()
	self:drawBossHealth()
	self:drawGotoDoorText()

	self:drawSacrificeSelection()
	self:drawGameOverScreen()
	self:drawEndScreen()
end


function file:init()
	Hud = Hud:new()
end

function file:update()
end

function file:draw()
end

-- [[ RETURN ]] --
return file