local file = {priority = 0}

Player = {}

function Player:new(pos)
	local self = Class:init(Player,Entity,{pos})

	self.dashTimer = 0
	self.dashTime = 0.35
	self.dashSpeed = 150
	self.respawnPoint = {30,30}
	self.spawnPoint = self.pos
	self.currentRun = 1

	self.sprite = Sprite:new({0,0})
	self.sprite:setParent(self)
	self.idleAnim = Animation:new("assets/player/idle.png",0.5)
	self.walkAnim = Animation:new("assets/player/walk.png",0.5)
	self.airUpAnim = Animation:new("assets/player/airUp.png",0.5)
	self.airDownAnim = Animation:new("assets/player/airDown.png",0.5)
	self.attackAnim = Animation:new("assets/player/attack.png",0.3)
	self.attack2Anim = Animation:new("assets/player/attack2.png",0.3)
	self.attackAirAnim = Animation:new("assets/player/attackAir.png",0.3)
	self.attack2AirAnim = Animation:new("assets/player/attack2Air.png",0.3)
	self.dashAnim = Animation:new("assets/player/dash.png",self.dashTime)
	self.anim = self.idleAnim

	self.attackSound = la.newSource("assets/audio/melee.wav","static")
	self.dashSound = la.newSource("assets/audio/dash.wav","static")
	self.stepSound = la.newSource("assets/audio/footstep.wav","static")
	self.hitSound = la.newSource("assets/audio/hit.wav","static")

	self.speed = 70
	self.gravity = 950
	self.jumpVelocity = 82
	self.verticalVelocity = 0
	self.terminalVelocity = 500
	self.radius = 3
	self.flip = 1

	self.maxHealth = 5
	self.health = self.maxHealth

	self.keys = 0

	self.attackDamage = 1
	self.attackTimer = 0
	self.attacks = 0
	self.maxAttackTimer = 0.2
	self.attackLurchSpeed = 40
	self.attackComboTime = -0.5
	self.hitEnemyTimer = 0

	self.invulnerableTimer = 0
	self.sacrifices = 0

	self.stepTimer = 0

	self.sprite.order = 1

	return self
end


function Player:respawn()
	Game.state = "playing"
	self:setPos(self.respawnPoint)

	AudioHandler.boss:stop()

	for j = 1,10 do
		for i, e in pairs(Enemy.list) do
			if e.isBoss then
				e.state = "sleep"
				e:update()
				e:destroy()
				e.class:new(e.origin)
			end
		end
	end
end

function Player:damage(damage)
	if self.invulnerableTimer <= 0 and self.dashTimer <= 0 then
		self.health = self.health - (damage or 1)
		self.invulnerableTimer = 1.5

		Utils.playSound(self.hitSound)

		if self.health <= 0 then
			Game.state = "gameover"
		end

		Juice:tilt(rad(7))
		Juice:freeze(1/30)
	end
end


function Player:setSpriteImg()
	self.sprite.img = self.anim:getFrame()
	self.sprite.sx = self.flip

	self.anim = self.idleAnim
end

function Player:control()
	if self.attackTimer <= 0 and self.dashTimer <= 0 then
		if Controller:isMovingLeft() then
			if World:getCell(self.pos[1],self.pos[2]+1).solid then
				self.anim = self.walkAnim
			end
			self:move({-self.speed*gdt,0})
			self.flip = -1
			self.stepTimer = self.stepTimer + gdt
		end
		if Controller:isMovingRight() then
			if World:getCell(self.pos[1],self.pos[2]+1).solid then
				self.anim = self.walkAnim
			end
			self:move({self.speed*gdt,0})
			self.flip = 1
			self.stepTimer = self.stepTimer + gdt
		end

		if self.stepTimer > 0.25 and World:getCell(self.pos[1],self.pos[2]+2).solid then
			self.stepTimer = 0
			Utils.playSound(self.stepSound,0.5,1+random()*rsign()*0.1)
		end
	end
end

function Player:fall()
	if self.attackTimer > 0 then
		self.verticalVelocity = 0
	else
		self.verticalVelocity = min(self.terminalVelocity, self.verticalVelocity + self.gravity * gdt * 0.2)
		self:move({0,self.verticalVelocity*gdt})
	end

	if self.verticalVelocity > 15 then
		self.anim = self.airDownAnim
	end
	if self.verticalVelocity < -15 then
		self.anim = self.airUpAnim
	end

	if World:getCell(self.pos).solid then
		self.verticalVelocity = 0
	end
end

function Player:jump()
	if Controller:isJumping() and World:getCell(self.pos[1],self.pos[2]+2).solid then
		self.verticalVelocity = -self.jumpVelocity
		self:move(0,-1)
	end
end

function Player:dash()
	self.dashTimer = self.dashTimer - gdt
	if Controller:isDashing() and self.dashTimer < -0.1 then
		self.dashTimer = self.dashTime
		Utils.playSound(self.dashSound,1,1+random()*rsign()*0.1)
	end

	if self.dashTimer > 0 then
		self.anim = self.dashAnim
		local speed = self.dashSpeed*((self.dashTimer/self.dashTime)^0.5+0.2)
		self:move(speed*self.flip*gdt,0)

		local squish = ((self.dashTimer/self.dashTime)^0.5)*0.4
		self.sprite.sx = 1+squish
		self.sprite.sy = 1-squish
	end
end

function Player:attack()
	self.attackTimer = self.attackTimer - gdt
	self.hitEnemyTimer = self.hitEnemyTimer - gdt
	if self.attackTimer < self.attackComboTime then
		self.attacks = 0
	end

	if self.attackTimer > -0.25 then
		if World:getCell({self.pos[1],self.pos[2]+1}).solid then
			self.anim = self.attack2Anim
			if self.attacks == 2 then
				self.anim = self.attackAnim
			end
		else
			self.anim = self.attack2AirAnim
			if self.attacks == 2 then
				self.anim = self.attackAirAnim
			end
		end
	end

	local lurch = 1
	if self.hitEnemyTimer > 0 then
		lurch = 0
	end
	if self.attacks == 3 then
		lurch = lurch * 2.5
	end
	self:move(max(0, self.attackTimer/self.maxAttackTimer) * self.flip * self.attackLurchSpeed * gdt * lurch,0)

	if Controller:isAttacking() and self.attackTimer <= 0 and self.attacks < 3 then
		self.attackTimer = self.maxAttackTimer

		Utils.playSound(self.attackSound,1,1+random()*rsign()*0.1)
		Juice:shake(0.7)

		local xDist = 22 * self.flip
		local yDist = 2.5
		local x,y = self.pos[1],self.pos[2]-5
		local lifeSpan = 0.3

		if self.attacks == 2 then
			xDist = xDist * 1.5
			lifeSpan = 0.5
			Juice:shake(0.9)
		end

		for a = -rad(90),rad(90),rad(1) do
			local size = abs( (Utils.boolToNumb(Utils.isEven(self.attacks-1)) - (a + rad(90))/rad(180)) * 2.5 ) + 0.5
			local pos = {x+cos(a)*xDist-self.flip*2,y+sin(a)*yDist}

			local color = {1,1,1}
			if a < 0 then
				color = {0.65,0.75,0.85}
			end

			local particle = Particle:new(pos,size,color,lifeSpan)
			particle.order = Player.sprite.order + sin(a)*0.1

			for i, e in pairs(Enemy.list) do
				if math.dist(e.pos[1],e.pos[2]-e.radius, pos[1],pos[2]) < e.radius + size + 2 then
					e:damage(self.attackDamage)
					self.hitEnemyTimer = self.attackTimer
				end
			end
			for i, p in pairs(Projectile.list) do
				if math.dist(p.pos[1],p.pos[2], pos[1],pos[2]) < p.radius + size + 2 then
					p:parry()
				end
			end
		end

		self.attacks = self.attacks + 1
	end
end

function Player:separateFromWorld()
	local pos = {self.pos[1],self.pos[2]-self.radius-1}
	local step = 0.02
	local m = {0,0}

	for i=1,10 do
		for a=0,rad(360),rad(10) do
			local x,y = pos[1]+cos(a)*self.radius,pos[2]+sin(a)*self.radius
			local cell = World:getCell(x,y)
			if cell.solid then
				m[1] = m[1] + cos(a+math.pi)*step
				m[2] = m[2] + sin(a+math.pi)*step
			end
		end
	end
	self:move(m)
end

function Player:setCamera()
	local pos = {self.pos[1],self.pos[2]-self.radius}

	Camera:setChasePos({
		(floor(pos[1]/Screen.resolution[1])+0.5)*Screen.resolution[1],
		(floor(pos[2]/Screen.resolution[2])+0.5)*Screen.resolution[2]
	})
end

function Player:updateHit()
	self.invulnerableTimer = max(0, self.invulnerableTimer - gdt)
end

function Player:update()
	if Game.state == "playing" then
		self:setSpriteImg()
		self:control()
		self:fall()
		self:jump()
		self:dash()
		self:attack()
		self:separateFromWorld()
		self:setCamera()
		self:updateHit()
	end
end



function Player:destroy()
	Entity.destroy(self)
	removeFromTable(Player.list,self)
end

function file:init()
	Player = Player:new({100,100})
	--Player = Player:new({1448,296})
	--Player = Player:new({100,500})
end

function file:update()
	if gdt == time then
		--Player:setPos({100,Screen.resolution[2]*4-50})
		Player:setCamera()
		Camera:setPos(Camera.chasePos)
	end
end

function file:draw()
end

-- [[ RETURN ]] --
return file