local file = {priority = 0}

Projectile = {list = {}}

function Projectile:new(pos,radius,dir)
	local self = Class:init(Projectile,Entity,{pos})

	self.radius = radius or 2
	self.dir = dir or {0,0}
	self.damage = 1
	self.speed = 1
	self.particleTimer = 0
	self.particleTimerMax = 0.02
	self.particleColor = {5/8,5/8,5/8}

	self.order = 1.1

	table.insert(Projectile.list,self)
	return self
end

function Projectile:parry()
	local p = Particle:new(self.pos,3+self.radius*2,{7/8,7/8,7/8},0.2)
	p.drawMode = "line"

	for i=1,40 do
		local a = random()*rad(360)
		local speed = random()*150
		local particle = Particle:new(self.pos,2,{6/8,6/8,6/8},0.3,150,{cos(a)*speed,sin(a)*speed-speed*0.2})
		particle.order = 0.8
	end

	self:destroy()
end


function Projectile:moveAround()
	self:move(self.dir[1]*gdt*self.speed*SacrificeHandler.projectileSpeed, self.dir[2]*gdt*self.speed*SacrificeHandler.projectileSpeed)
end

function Projectile:handleCollision()
	if World:getCell(self.pos).solid then
		self:destroy()
	end

	if math.dist(self.pos[1],self.pos[2], Player.pos[1],Player.pos[2]-Player.radius) < Player.radius + self.radius - 2 then
		Player:damage(self.damage)
	end
end

function Projectile:update()
	self:handleCollision()
	self:moveAround()
end

function Projectile:spawnTrailParticles()
	self.particleTimer = self.particleTimer - gdt
	if self.particleTimer < 0 then
		self.particleTimer = self.particleTimerMax

		Particle:new(self.pos,self.radius,self.particleColor,15/self.speed)
	end
end

function Projectile:draw()
	self:spawnTrailParticles()

	lg.setColor(5/8,5/8,5/8)
	lg.circle("fill", self.pos[1],self.pos[2],self.radius)

	lg.setColor(6/8,6/8,6/8)
	local minR = self.radius * 0.7
	local a = math.angle(0,0, self.dir[1],self.dir[2])
	lg.circle("fill", self.pos[1]+cos(a)*(self.radius-minR) ,self.pos[2]+sin(a)*(self.radius-minR), minR)
end


function Projectile:destroy()
	Entity.destroy(self)
	removeFromTable(Projectile.list,self)
end

function file:init()
	Projectile.impactSound = la.newSource("assets/audio/impact.wav","static")
	Projectile.impactSound:setVolume(0.5)
end

function file:update()
end

function file:draw()
end

-- [[ RETURN ]] --
return file