local file = {priority = 0}

World = {}

local function fillCell(color, pos)
	local cell = {}
	local noiseScale = 0.1
	local noiseRandomness = 1.7
	local grassScale = 0.05

	cell.deter = love.math.noise((pos[1]+random()*rsign()*noiseRandomness)*noiseScale,(pos[2]+random()*rsign()*noiseRandomness)*noiseScale)
	cell.grassiness = love.math.noise(pos[1]*grassScale,pos[2]*grassScale)
	if Utils.isColor(color, {1,1,1}) then
		cell.solid = true
	end
	if Utils.isColor(color, {0,69/255,69/255}) then
		cell.lock = true
		cell.solid = true
	end
	if Utils.isColor(color, {56/255,178/255,178/255}) then
		Spawner:new({pos[1]*8+4,pos[2]*8}, Grunt)
	end
	if Utils.isColor(color, {109/255,41/255,178/255}) then
		Spawner:new({pos[1]*8+4,pos[2]*8}, Bat)
	end
	if Utils.isColor(color, {79/255,191/255,42/255}) then
		Spawner:new({pos[1]*8+4,pos[2]*8}, Sentry)
	end
	if Utils.isColor(color, {200/255,200/255,100/255}) then
		Checkpoint:new({pos[1]*8+4,pos[2]*8})
	end

	if Utils.isColor(color, {0.5,0.5,0.5}) then
		cell.bgSolid = true
	end
	return cell
end

local function createArray(data)
	local array = {}
	for x=1,data:getWidth() do
		array[x] = {}
		for y=1,data:getHeight() do
			local r,g,b,a = data:getPixel(x-1,y-1)
			array[x][y] = fillCell({r,g,b,a}, {x,y})
		end
	end
	return array
end

function World:new(pos)
	local self = Class:init(World,Dynamic)

	self.levelData = Pencil.canvasFromImage(lg.newImage("objects/level.png")):newImageData()
	self.array = createArray(self.levelData)

	self.locked = false
	self.lockImg = lg.newImage("assets/tiles/lock.png")

	self.grassTiles = {
		lg.newImage("assets/tiles/grass1.png"),
		lg.newImage("assets/tiles/grass2.png"),
	}

	self.tiles = {
		lg.newImage("assets/tiles/tile1.png"),
		lg.newImage("assets/tiles/tile2.png"),
		lg.newImage("assets/tiles/tile3.png"),
		lg.newImage("assets/tiles/tile4.png"),
		lg.newImage("assets/tiles/tile5.png"),
	}

	self.order = 0.95

	return self
end

function World:getCell(x,y)
	if type(x) == "table" then
		x,y = x[1],x[2]
	end

	x = clamp(1,floor(x/8)+1,#self.array)
	y = clamp(1,floor(y/8)+1,#self.array[1])

	return self.array[x][y]
end


function World:lockUnlock()
	self.locked = false
	for i,e in pairs(Enemy.list) do
		if e.isBoss and e.state ~= "sleep" then
			self.locked = true
		end
	end

	local xw,yw = floor(Camera.pos[1]/8),floor(Camera.pos[2]/8)
	local spaceX,spaceY = 21,14

	for i = max(1,xw-spaceX), min(xw+spaceX,#self.array[1]-2) do
		for j = max(1,yw-spaceY), min(yw+spaceY,#self.array-2) do
			local cell = self.array[i][j]
			if self.locked and cell.lock then
				cell.solid = true
			end
			if not self.locked and cell.lock then
				cell.solid = false
			end
			self.array[i][j] = cell
		end
	end

end


function World:update()
	self:lockUnlock()
end


function World:drawTiles()
	local xw,yw = floor(Camera.pos[1]/8),floor(Camera.pos[2]/8)
	local spaceX,spaceY = 21,14

	lg.setColor(1,1,1)
	for i = max(1,xw-spaceX), min(xw+spaceX,#self.array[1]-2) do
		for j = max(1,yw-spaceY), min(yw+spaceY,#self.array-2) do
			local cell = self.array[i][j]
			local x,y = (i-1)*8,(j-1)*8
			if cell.solid then
				if cell.lock then
					lg.setColor(1,1,1)
					lg.draw(self.lockImg,x,y)
				else
					lg.setColor(1,1,1)
					lg.draw(self.tiles[ceil(cell.deter*#self.tiles)],x,y)
					if not self.array[i][max(1,j-1)].solid then
						if cell.grassiness > 0.35 then
							lg.draw(self.grassTiles[1],x,y)
						end
						if cell.grassiness > 0.65 then
							lg.draw(self.grassTiles[2],x,y)
						end
					end
				end
			end
		end
	end
end

function World:draw()
	self:drawTiles()
end


function file:init()
	World = World:new()
end

function file:update()
end

function file:draw()
end

-- [[ RETURN ]] --
return file