local file = {priority = 1}

Boss1 = {}

function Boss1:new(pos)
	local self = Class:init(Boss1,Enemy,{pos, 7, 30})

	self.origin = self.pos
	self.flip = 1
	self.isBoss = true
	self.anim = Animation:new("assets/enemies/boss1Idle.png",0.6)

	self.state = "sleep"
	self.nextState = "n/a"
	self.timer = 0
	self.timerSpeed = 0.75
	self.secTimer = 0
	self.wakeDist = 100
	self.target = self.origin
	self.speed = 40

	self.particleTimer = 0
	self.enragedTimer = 0

	self.sprite.order = 1.5

	return self
end

function Boss1:isAtTarget()
	return math.dist(self.pos[1],self.pos[2], self.target[1],self.target[2]) < 1
end


function Boss1:runStateMachine()
	self.timer = self.timer + gdt * self.timerSpeed * SacrificeHandler.bossTimerSpeed
	self.secTimer = self.secTimer + gdt * self.timerSpeed * SacrificeHandler.bossTimerSpeed
	
	if self.state == "sleep" then
		self:move(0,sin(time)*gdt*4)
		if self:dist(Player) < self.wakeDist and self:onScreen() then
			AudioHandler.boss:play()
			self.state = "gotoOrigin"
			self.nextState = "sweep"
		end
	end
	if self.state == "gotoOrigin" then
		self.speed = 120
		self.target = self.origin

		if math.dist(self.pos[1],self.pos[2], self.origin[1],self.origin[2]) < 2 then
			self.state = self.nextState
			self.timer = 0
		end
	end

	if self.state == "missiles" then
		self.state = "gotoOrigin"
		self.nextState = "missilesFire"
	end
	if self.state == "missilesFire" then
		self.speed = 60
		local a = time*1.5
		local dist = 30
		self.target = {self.origin[1]+cos(a)*dist, self.origin[2]+sin(a)*dist}

		if self.secTimer > 0.6 then
			self.secTimer = 0
			Utils.playSound(self.projectileSound)
			for i = 0,rad(360),rad(30) do
				Boss1Missile:new(self:getCenter(),{cos(i),sin(i)})
			end
		end

		if self.timer > 6 then
			self.state = "gotoOrigin"
			self.flip = rsign()
			self.nextState = Utils.randomItem({"smash","sweep"})
		end
	end

	if self.state == "smash" then
		self.speed = 150
		self.target = {self.origin[1]-120.5*self.flip,self.origin[2] + sin(time*7)*2}

		if self.timer > 2 then
			self.state = "smashFall"
			self.timer = 0
		end
	end
	if self.state == "smashFall" then
		self.speed = 200
		self.target = {self.origin[1]-120.5*self.flip,self.origin[2]+100.5-16}

		if self:isAtTarget() then
			self.timer = 0
			self.state = "smashWinddown"
			Utils.playSound(self.impactSound)

			Utils.playSound(self.projectileSound)

			for i = self.target[2]-5,self.target[2]-40,-10 do
				Boss1SmashMissile:new({self.pos[1],i},{self.flip,0})
			end

			Juice:shake(3)
		end
	end
	if self.state == "smashWinddown" then
		self.speed = 0
		if self.timer > 5 then
			self.state = "gotoOrigin"
			self.flip = rsign()
			self.nextState = Utils.randomItem({"missiles","sweep"})
		end
	end

	if self.state == "sweep" then
		self.target = {self.origin[1],self.origin[2]-30 + sin(time*6)*5}

		if self.timer > 2 then
			self.timer = 0
			self.state = "sweepActive"
		end
	end
	if self.state == "sweepActive" then
		self.target = {self.origin[1]+sin(self.timer*1)*130,self.origin[2]-30}

		if self.secTimer > 0.1 then
			self.secTimer = 0
			Boss1DropMissile:new(self:getCenter(),{0,1})
		end

		if self.timer > 6 then
			self.timer = 0
			self.state = "sweepSmashWindup"
		end
	end
	if self.state == "sweepSmashWindup" then
		self.target = {self.origin[1],self.origin[2]-10 + sin(time*10)*2}

		if self.timer > 1 then
			self.timer = 0
			self.state = "sweepSmashFall"
		end
	end
	if self.state == "sweepSmashFall" then
		self.speed = 200
		self.target = {self.target[1],self.origin[2]+100.5-16}

		if self:isAtTarget() then
			Utils.playSound(self.impactSound)
			Utils.playSound(self.projectileSound)
			self.timer = 0
			self.state = "sweepSmashFallWinddown"

			for i = self.target[2]-5,self.target[2]-40,-10 do
				Boss1Missile:new({self.pos[1],i},{1,0})
				Boss1Missile:new({self.pos[1],i},{-1,0})
			end

			Juice:shake(4)
		end
	end
	if self.state == "sweepSmashFallWinddown" then
		self.speed = 0
		if self.timer > 5 then
			self.state = "gotoOrigin"
			self.flip = rsign()
			self.nextState = Utils.randomItem({"missiles","smash"})
		end
	end

end

function Boss1:chaseTarget()
	local a = math.angle(self.pos[1],self.pos[2], self.target[1]+.1,self.target[2]+.1)
	self:move(cos(a)*self.speed*gdt, sin(a)*self.speed*gdt)
end

function Boss1:enrage()
	if self.health < self.maxHealth*0.35 and not self.enraged then
		self.enraged = true

		self.timerSpeed = 0.85

		Juice:shake(3)
	end

	if self.enraged then
		self.enragedTimer = self.enragedTimer + gdt
		if self.enragedTimer > 0.2 then
			self.enragedTimer = 0
			local p = Particle:new(self:getCenter(),25,{3/8,3/8,3/8},0.5)

			p.drawMode = "line"
		end
	end
end

function Boss1:spawnParticles()
	local maxTimer = 0.02
	self.particleTimer = self.particleTimer + gdt
	if self.particleTimer > maxTimer then
		self.particleTimer = 0

		local radius = random()*2+2
		local pos = {self:getCenter()[1]+random()*rsign()*(self.radius-radius)*0.5,self:getCenter()[2]+random()*rsign()*(self.radius-radius)*0.5}
		Particle:new(pos,radius,{5/8,5/8,5/8},1)

		if self.enraged then
			radius = random()*3+4
			pos = {self:getCenter()[1]+random()*rsign()*self.radius*1.15,self:getCenter()[2]+random()*rsign()*self.radius*1.15}
			local p = Particle:new(pos,radius,{4/8,4/8,4/8},1)
			p.order = 0.7
		end
	end
end

function Boss1:update()
	Enemy.update(self)

	self:runStateMachine()
	self:chaseTarget()
	self:spawnParticles()
	self:enrage()
end


function file:init()
end

function file:update()
	if gdt == time then
	end
end

function file:draw()
end

-- [[ RETURN ]] --
return file