local file = {priority = 1}

Boss2 = {}

function Boss2:new(pos)
	local self = Class:init(Boss2,Enemy,{pos, 4, 30})

	self.origin = self.pos
	self.flip = 1
	self.isBoss = true
	self.idleAnim = Animation:new("assets/enemies/boss2Idle.png",0.6)
	self.walkAnim = Animation:new("assets/enemies/boss2Walk.png",1)
	self.airUpAnim = Animation:new("assets/enemies/boss2AirUp.png",1)
	self.airDownAnim = Animation:new("assets/enemies/boss2AirDown.png",1)
	self.airRecoverAnim = Animation:new("assets/enemies/boss2Recover.png",1)
	self.punchAnim = Animation:new("assets/enemies/boss2Punch.png",1)
	self.punchWindupAnim = Animation:new("assets/enemies/boss2PunchWindup.png",1)
	self.givenUpAnim = Animation:new("assets/enemies/boss2GivenUp.png",1)
	self.anim = self.idleAnim

	self.state = "sleep"
	self.nextState = "n/a"
	self.timer = 0
	self.timerSpeed = 0.75
	self.secTimer = 0
	self.wakeDist = 100
	self.target = self.origin
	self.speed = 40

	self.particleTimer = 0
	self.enragedTimer = 0

	self.sprite.order = 1.5

	self:move(100,0)

	return self
end

function Boss2:isAtTarget(n)
	return math.dist(self.pos[1],self.pos[2], self.target[1],self.target[2]) < (n or 1)
end


function Boss2:runStateMachine()
	self.timer = self.timer + gdt * self.timerSpeed * SacrificeHandler.bossTimerSpeed
	self.secTimer = self.secTimer + gdt * self.timerSpeed * SacrificeHandler.bossTimerSpeed

	if self.state == "sleep" then
		self.speed = 0
		if self:dist(Player) < self.wakeDist and self:onScreen() then
			AudioHandler.boss:play()
			self.state = "walk"
			self.timer = 0
		end
	end

	if self.state == "walk" then
		self.speed = 15
		self.target = {Player.pos[1],self.origin[2]}
		self.flip = sign(Player.pos[1]-self.pos[1])
		self.anim = self.walkAnim

		if self.timer > 1 and self:dist(Player) < 20 then
			self.timer = 0
			self.state = Utils.randomItem({"punch","punch","punch","smash","smash","smashPunch"})
		end

		if self.timer > 3 then
			self.timer = 0
			self.state = Utils.randomItem({"smash","smashPunch"})
		end
	end

	if self.state == "smash" then
		if self.enraged then
			self.smashes = 10
		else
			self.smashes = 3
		end
		self.state = "smashLaunch"
	end
	if self.state == "smashLaunch" then
		self.speed = 300
		self.target = {self.pos[1],self.origin[2]-90}
		
		self.flip = sign(Player.pos[1]-self.pos[1])
		self.anim = self.airUpAnim

		if self:isAtTarget(2) then
			self.state = "smashWindup"
			self.timer = 0
		end
	end
	if self.state == "smashWindup" then
		self.speed = 0

		if self.timer < 0.45 then
			self.flip = sign(Player.pos[1]-self.pos[1])
			self.smashAng = math.angle(self.pos[1],self.pos[2], Player.pos[1],Player.pos[2])
			self.anim = self.airDownAnim

			self.watchoutSound:play()
		end

		if self.timer > 0.6 then
			self.state = "smashFall"
			self.timer = 0
		end
	end
	if self.state == "smashFall" then
		self.speed = 500
		self.target = {self.pos[1]+cos(self.smashAng),self.pos[2]+sin(self.smashAng)}

		if self:collides() then
			Utils.playSound(self.impactSound)
			self.pos[2] = self.origin[2]
			self.state = "smashRecover"
			self.timer = 0
			Juice:shake(2)

			if self.enraged then
				Boss2Shockwave:new(self.pos,{1,0})
				Boss2Shockwave:new(self.pos,{-1,0})
				if self.smashes == 1 then
					self.timer = -1000000
				end
			end
		end
	end
	if self.state == "smashRecover" then
		self.speed = 0
		self.anim = self.airRecoverAnim 
		if self.enraged then
			self.anim = self.givenUpAnim
		end

		if self.smashes > 1 and self.timer > 0.3 then
			self.smashes = self.smashes - 1
			self.timer = 0
			self.state = "smashLaunch"
		end

		if self.timer > 2 then
			self.state = "walk"
			self.timer = 0
		end
	end

	if self.state == "punch" then
		self.watchoutSound:play()
		self.speed = 0
		self.anim = self.punchWindupAnim
		self.flip = sign(Player.pos[1]-self.pos[1])

		self.target = {self.pos[1]+self.flip*1000,self.pos[2]}

		if self.timer > 0.7 then
			self.state = "punchFlying"
			self.timer = 0
		end
	end
	if self.state == "punchFlying" then
		self.anim = self.punchAnim
		local punchTime = 0.5

		self.speed = (1-self.timer/punchTime) * 220

		if self.timer > punchTime or self:collides({0,-3}) then
			self.state = "walk"
			self.timer = 0
		end
	end

	if self.state == "smashPunch" then
		self.anim = self.airRecoverAnim
		self.speed = 0

		if self.timer > 1 then
			self.timer = 0
			self.state = "smashPunchLaunch"
		end
	end
	if self.state == "smashPunchLaunch" then
		self.anim = self.airUpAnim
		self.speed = 400
		self.flip = sign(Player.pos[1]-self.pos[1])
		self.target = {self.origin[1],self.origin[2]-80}

		if self:isAtTarget(2) then
			self.timer = 0
			self.state = "smashPunchWindup"
		end
	end
	if self.state == "smashPunchWindup" then
		self.anim = self.airDownAnim
		self.speed = 0

		if self.timer < 1.7 then
			self.flip = sign(Player.pos[1]-self.pos[1])
			self.smashAng = math.angle(self.pos[1],self.pos[2], Player.pos[1],Player.pos[2])
		end

		if self.timer < 1 then
			self:move(sign(sin(time*30))*20*gdt,0)
		else
			self:move(sign(sin(time*50))*40*gdt,0)
		end

		if self.timer > 1.8 then
			self.timer = 0
			self.state = "smashPunchFall"
		end
	end
	if self.state == "smashPunchFall" then
		self.speed = 500
		self.target = {self.pos[1]+cos(self.smashAng),self.pos[2]+sin(self.smashAng)}

		if self:collides() then
			Utils.playSound(self.impactSound)
			self.state = "smashPunchPunchWindup"
			self.timer = 0
			self:setPos({self.pos[1],self.origin[2]})

			Boss2Shockwave:new(self.pos,{1,0})
			Boss2Shockwave:new(self.pos,{-1,0})
		end
	end
	if self.state == "smashPunchPunchWindup" then
		self.speed = 0
		self.flip = sign(Player.pos[1]-self.pos[1])
		self.anim = self.punchWindupAnim
		self.target = {self.pos[1]+self.flip*1000,self.pos[2]}

		if self.timer > 1 then
			self.state = "smashPunchPunch"
			self.timer = 0
		end
	end
	if self.state == "smashPunchPunch" then
		self.anim = self.punchAnim
		local punchTime = 0.5

		self.speed = (1-self.timer/punchTime) * 300

		if self.timer > punchTime or self:collides({0,-3}) then
			self.state = "smashPunchRecover"
			self.timer = 0
		end
	end
	if self.state == "smashPunchRecover" then
		self.speed = 0
		self.anim = self.airRecoverAnim
		if self.timer > 4 then
			self.state = "walk"
			self.timer = 0
		end
	end

	if self.state == "justEnraged" then
		self.anim = self.airUpAnim
		self.target = {self.origin[1],self.origin[2]-80}
		self.speed = 500
		if self:isAtTarget(2) then
			self.state = "raging"
			self.timer = 0
		end
	end
	if self.state == "raging" then
		self:move(sign(sin(time*10))*50*gdt,0)

		if self.timer > 5 then
			self.state = "smash"
			self.timer = 0
		end
	end

end

function Boss2:chaseTarget()
	local a = math.angle(self.pos[1],self.pos[2], self.target[1]+.1,self.target[2]+.1)
	self:move(cos(a)*self.speed*gdt, sin(a)*self.speed*gdt)
end

function Boss2:enrage()
	if self.health < self.maxHealth*0.1 and not self.enraged then
		self.enraged = true

		self.timerSpeed = 0.8
		self.state = "justEnraged"
		self.timer = 0

		Juice:shake(3)
	end

	if self.enraged then
		self.enragedTimer = self.enragedTimer + gdt
		if self.enragedTimer > 0.01 then
			self.enragedTimer = 0

			local radius = random()*2+2
			local pos = {self:getCenter()[1]+random()*rsign()*self.radius*0.4,self:getCenter()[2]+random()*rsign()*self.radius*0.4}
			local p = Particle:new(pos,radius,{4/8,4/8,4/8},1)
			p.order = 0.8
			radius = random()*3+4
			pos = {self:getCenter()[1]+random()*rsign()*self.radius*1.15,self:getCenter()[2]+random()*rsign()*self.radius*1.15}
			local p = Particle:new(pos,radius,{3/8,3/8,3/8},1)
			p.order = 0.7
		end
	end
end

function Boss2:spawnParticles()
end

function Boss2:update()
	Enemy.update(self)

	self:runStateMachine()
	self:chaseTarget()
	self:spawnParticles()
	self:enrage()
end


function file:init()
end

function file:update()
	if gdt == time then
		
	end
end

function file:draw()
end

-- [[ RETURN ]] --
return file