local file = {priority = 1}

Boss3 = {}

function Boss3:new(pos)
	local self = Class:init(Boss3,Enemy,{pos, 7, 30})

	self.origin = self.pos
	self.flip = 1
	self.isBoss = true
	self.anim = Animation:new("assets/enemies/boss3Idle.png",0.6)

	self.state = "sleep"
	self.nextState = "n/a"
	self.timer = 0
	self.timerSpeed = 0.7
	self.secTimer = 0
	self.wakeDist = 100
	self.target = self.origin
	self.speed = 40
	self.speedMult = 1

	self.particleTimer = 0
	self.enragedTimer = 0

	self.sprite.order = 1.5

	return self
end

function Boss3:isAtTarget()
	return math.dist(self.pos[1],self.pos[2], self.target[1],self.target[2]) < 1
end


function Boss3:runStateMachine()
	self.timer = self.timer + gdt * self.timerSpeed * SacrificeHandler.bossTimerSpeed
	self.secTimer = self.secTimer + gdt * self.timerSpeed * SacrificeHandler.bossTimerSpeed
	
	if self.state == "sleep" then
		self:move(0,sin(time)*gdt*4)
		if self:dist(Player) < self.wakeDist and self:onScreen() then
			AudioHandler.boss:play()
			--self.state = "volcano"
			self.state = Utils.randomItem({"volcano","pincer","spin"})
		end
	end
	if self.state == "gotoOrigin" then
		self.speed = 100
		self.target = self.origin

		if math.dist(self.pos[1],self.pos[2], self.origin[1],self.origin[2]) < 2 then
			self.state = self.nextState
			self.timer = 0
		end
	end

	if self.state == "spin" then
		self.flip = rsign()
		self.state = "spinToPos"
	end
	if self.state == "spinToPos" then
		self.speed = 100
		self.target = {self.origin[1]+100*self.flip,self.origin[2]+60}

		if self:isAtTarget() then
			self.target = {self.origin[1]+100*self.flip*-1,self.origin[2]+60}
			self.timer = 0
			self.state = "spinSweep"
		end
	end
	if self.state == "spinSweep" then
		self.speed = 40
		local ang = time*rad(130)

		if self.secTimer > 0.075 then
			self.secTimer = 0
			Boss3Missile:new(self:getCenter(),{cos(ang),sin(ang)})
		end

		if self:isAtTarget() then
			self.state = "gotoOrigin"
			self.nextState = Utils.randomItem({"volcano","pincer"})
		end
	end

	if self.state == "volcano" then
		self.target = self.origin
		self.state = "volcanoWindup"

		self.aStep = rad(20)
		self.pDist = 35
		self.fireCount = 3
		self.secTimer = 0
		self.timer = 0
		self.volPos = Player.pos
	end
	if self.state == "volcanoWindup" then
		self.speed = 50
		self.target = {self.origin[1]+sin(time)*10,self.origin[2]}

		if self.secTimer > 2-1 and self.fireCount > 0 then
			self.secTimer = -100
			for a = self.aStep,rad(360),self.aStep do
				self.volPos = {Player.pos[1],Player.pos[2]}
				local pos = {self.volPos[1]+cos(a)*self.pDist,self.volPos[2]+sin(a)*self.pDist}
				Particle:new(pos,3,{5/8,5/8,5/8},1)
			end
		end

		if self.timer > 2 then
			self.timer = 0
			self.secTimer = 0
			self.state = "volcanoWinddown"
		end
	end
	if self.state == "volcanoWinddown" then
		self.speed = 60
		self.target = {self.origin[1]+random()*rsign()*20,self.origin[2]+random()*rsign()*20}

		if self.fireCount > 0 then
			self.timer = 0
			self.fireCount = self.fireCount - 1
			for a = self.aStep,rad(360),self.aStep do
				local pos = {self.volPos[1]+cos(a)*self.pDist,self.volPos[2]+sin(a)*self.pDist}
				Boss3Missile:new(pos, {cos(a-math.pi),sin(a-math.pi)})
			end
			Utils.playSound(self.projectileSound,1)
			self.secTimer = 0
			self.state = "volcanoWindup"
		end

		if self.timer > 0.75 then
			self.speed = 200
			self.target = {self.pos[1],self.pos[2]+200}

			if self:collides() then
				self.state = "volcanoRecover"
				self.timer = 0

				Juice:shake(3)
				Utils.playSound(self.impactSound)
				Utils.playSound(self.projectileSound)

				for a = self.aStep,rad(360),self.aStep do
					local pos = {self.pos[1],self.pos[2]-5}
					Boss3Missile:new(pos, {cos(a),sin(a)})
				end
			end
		end
	end
	if self.state == "volcanoRecover" then
		self.speed = 0

		if self.timer > 5 then
			self.state = "gotoOrigin"
			self.nextState = Utils.randomItem({"spin","pincer"})
		end
	end
	
	if self.state == "pincer" then
		local dist = 10
		local ang = time*rad(360)
		self.target = {self.origin[1]+cos(ang)*dist,self.origin[2]+sin(ang)*dist}

		if self.timer > 3.5 then
			self.timer = 0
			self.state = "pincerSmash"
			self.smashCount = 3
		end
	end
	if self.state == "pincerSmash" then
		self.speed = 175
		self.target = {self.origin[1]+50*self.flip,self.origin[2]+150}

		if self:collides() then
			self.timer = 0
			self.state = "pincerRecover"

			Juice:shake(2)
			Utils.playSound(self.impactSound,0.75)
			Utils.playSound(self.projectileSound,1)

			if self.smashCount == 1 then
				Boss3Mortar:new({Player.pos[1]-20,self.origin[2]-30},{0,0},6)
				Boss3Mortar:new({Player.pos[1]+20,self.origin[2]-30},{0,0},6)
			else
				Boss3Mortar:new({Player.pos[1],self.origin[2]-30},{0,0},6)
			end
			self.smashCount = self.smashCount - 1
		end
	end
	if self.state == "pincerRecover" then
		self.speed = 100
		self.target = {self.origin[1]+75*self.flip,self.origin[2]+60}

		if self.smashCount < 1 then
			self.timer = 0
			self.state = "pincerWinddown"
		end

		if self.timer > 2 then
			self.timer = 0
			self.state = "pincerSmash"
		end
	end
	if self.state == "pincerWinddown" then
		self.speed = 0
		if self.timer > 4 then
			self.state = "gotoOrigin"
			self.nextState = Utils.randomItem({"volcano","spin"})
		end
	end

end

function Boss3:chaseTarget()
	if not self:isAtTarget() then
		local a = math.angle(self.pos[1],self.pos[2], self.target[1]+.1,self.target[2]+.1)
		self:move(cos(a)*self.speed*self.speedMult*gdt, sin(a)*self.speed*self.speedMult*gdt)
	end
end

function Boss3:enrage()
	if self.health < self.maxHealth*0.35 and not self.enraged then
		self.enraged = true
		self.speedMult = 1.5
	end
end

function Boss3:spawnParticles()
	local maxTimer = 0.02
	self.particleTimer = self.particleTimer + gdt
	if self.particleTimer > maxTimer then
		self.particleTimer = 0

		local radius = random()*2+2
		local pos = {self:getCenter()[1]+random()*rsign()*(self.radius-radius)*0.5,self:getCenter()[2]+random()*rsign()*(self.radius-radius)*0.5}
		Particle:new(pos,radius,{5/8,5/8,5/8},1)

		if self.enraged then
			radius = random()*3+4
			pos = {self:getCenter()[1]+random()*rsign()*self.radius*1.15,self:getCenter()[2]+random()*rsign()*self.radius*1.15}
			local p = Particle:new(pos,radius,{4/8,4/8,4/8},1)
			p.order = 0.7
		end
	end
end

function Boss3:update()
	Enemy.update(self)

	self:runStateMachine()
	self:chaseTarget()
	self:spawnParticles()
	self:enrage()
end


function file:init()
end

function file:update()
	if gdt == time then
	end
end

function file:draw()
end

-- [[ RETURN ]] --
return file