local file = {priority = 1}

shader_screenSmooth = lg.newShader[[
	extern bool enabled;
	extern Image screenMap;
	extern Image ignoreMap;

	float noise(vec2 pos)
    {
    	return fract(sin(dot(pos, vec2(12.9898, 78.233))) * 43758.5453);
    }

    vec4 effect( vec4 color, Image texture, vec2 tc, vec2 sc ){
		
		vec4 pixel = Texel(texture, tc);
		vec4 scPixel = Texel(screenMap, tc);
		vec4 igPixel = Texel(ignoreMap, tc);
		float xMod = 9.0/16.0;

		// skip if smooth shaders are not enabled
		if (!enabled) {
			return pixel;
		}

		// skip ignored pixels
		if (igPixel.r > 0) {
			return pixel;
		}

		// vignette
		float vigEffect = 1;
		float vigDist = pow(distance(tc, vec2(0.5,0.5)),4)*vigEffect;
		pixel *= (1-vigDist);

		// ao
		if (scPixel.r == 0) {
			float aoDist = 0.045;
			float aoStep = aoDist/10;
			float aoEffect = 0.00425;
			vec4 aoMult = vec4(1,1,1,1);
			vec4 aoPixel;
			for (float x=-aoDist*xMod;x<aoDist*xMod;x+=aoStep) {
				for (float y=-aoDist;y<aoDist;y+=aoStep) {
					vec2 aoPos = vec2(x,y) + tc + vec2(noise(vec2(x,y)),noise(vec2(x*0.5,y*-0.3)))*aoStep;
					if (distance(aoPos,tc) <= aoDist) {
						aoPixel = Texel(screenMap, aoPos);
						if (aoPixel.r > 0) {
							aoMult.r -= aoEffect;
							aoMult.g -= aoEffect*0.75;
							aoMult.b -= aoEffect*0.5;
						}
					}
				}
			}
			pixel *= aoMult;
		}

		// boost colors
		pixel *= 1.05;

		// RETURN
		return pixel;

    }
]]

-- [[ RETURN ]] --
return file